==> 0.0.5.tar.gz: lib/API/PIA_API_Gridlist.dart <==
import 'package:dio/src/dio_error.dart';
import 'package:flutter/material.dart';
import 'package:pia_framework/API/PIA_API_Background.dart';
import 'package:pia_framework/API/PIA_API_Widget.dart';
import 'package:pia_framework/Variable/PIA_Global.dart';
import 'package:pia_framework/Lang/PIA_Lang.dart';

class PIA_API_Gridlist_Controller
{   PIA_API_GridList_State list_state;
    void reload(String url, Map<String,String> post)
      {  list_state.Url_reload(url,post); }
}


class PIA_API_Gridlist_CallBack
{    Widget Gridlist_item(int item,Map<String,dynamic> sdata,Function updatestate) {}

/* example for callback
 return new Padding(padding: EdgeInsets.all(0),
      child: new ListTile(
      //leading: PIA_Image("Asset/Image/company.png",50,50,0,BoxFit.cover),
      title: new Text(sdata["api"],style:new TextStyle(color: (sdata["isclick"] == "1" ) ? Colors.blue : Colors.pink)),
      subtitle: (sdata["isclick"] == "1") ? PIA_Image("Asset/Image/company.png",150,150,0,BoxFit.cover): null,
          onTap: (){
        if(sdata["isclick"] != "1")
          {  sdata["isclick"] = "1"; }
         else
          {  sdata["isclick"] = ""; }

        updatestate(null);
      },
    ));
*/
}

class PIA_API_GridList extends StatefulWidget
{  String url;
   Map<String,String> post;
   PIA_API_Gridlist_CallBack callback;
   PIA_API_Gridlist_Controller controller;

   int pagesize = 20;

   PIA_API_GridList({Key key, this.url,this.post,this.callback,this.pagesize,this.controller}) : super(key: key);

   @override
   State<StatefulWidget> createState() {
     // TODO: implement createState

     if(post == null)
       { post = Map<String,String>(); }

     if(pagesize == null)
       { pagesize = 20; }

     post["pageno"] = "1";
     post["pagesize"] = "$pagesize";

     print("PIA_TableList Build " + post.toString());

     var tablestate = PIA_API_GridList_State();
     controller.list_state = tablestate;
     return tablestate;
  }
}


class PIA_API_GridList_State  extends  State<PIA_API_GridList> implements PIA_API_CallBack{

  bool firstload = true;
  bool all_loaded = false;
  bool is_loading = false;
  bool is_error = false;

  ScrollController _controller;
  int pageno = 1;


  final items = new List<Map<String,dynamic>>();

  void initState() {
    _controller = ScrollController();
    _controller.addListener(_scrollListener);
  }

  _scrollListener() {
    if (_controller.offset >= _controller.position.maxScrollExtent &&
        !_controller.position.outOfRange) {
        setState(() {
          loadurl();
        });
    }
    if (_controller.offset <= _controller.position.minScrollExtent &&
        !_controller.position.outOfRange) {
    }
  }

  void Url_reload(String url,Map<String,String> post)
  {  PIA_API_Background.Stop();
  widget.post = post;
  widget.url = url;
  is_loading = false;
  reload();
  }


  void reload()
  { if(is_loading) return;

    setState(()
    { items.clear();
      firstload = true;
      all_loaded = false;
      is_loading = false;
      pageno = 1;

      widget.post["pageno"] = "$pageno";
      widget.post["pagesize"] = "${widget.pagesize}";

     });
  }

  void loadurl()
  {  print("Load URL ${is_loading}");

     if(!all_loaded && !is_loading) {
     is_loading = true;
     widget.post["pageno"] = "$pageno";
     widget.post["pagesize"] = "${widget.pagesize}";
     PIA_API_Background.Post(widget.url,widget.post,this);
    }
    else
     { print("All Data Loaded"); }
  }

  void update_state(Function itemfunction)
  {   setState(() {
         if(itemfunction != null)
           { itemfunction(); }
      });
  }

  @override
  void dispose() {
    super.dispose();
    PIA_API_Background.Stop();
    print("TableView dispose");
  }



  @override
  Widget build(BuildContext context) {
    print("Tablelist Build firstload $firstload "+ " post:" + widget.post.toString());

    if(firstload) {
        is_loading = true;
        return PIA_API_Widget(url:widget.url,post:widget.post,callback: this);
    }

    if(items.length > 0) {
      return
        new Stack(
          children: <Widget>[
               RefreshIndicator(
               onRefresh: (){ reload(); },
                child: GridView.builder(
                itemCount: items.length,
                 controller: _controller,
                 gridDelegate: new SliverGridDelegateWithFixedCrossAxisCount(crossAxisCount:2),
                  itemBuilder: (BuildContext context, int index) {
                      return widget.callback.Gridlist_item(index, items[index],update_state);
                  },
           )),
            Footer(),
          ]);
    }
    else {
      return Center(
          child: Column(
          mainAxisAlignment: MainAxisAlignment.center,
          crossAxisAlignment: CrossAxisAlignment.center,
          children: <Widget>[
             Text(tableview_nodata[PIA_Lang.lang_id],style:TextStyle(fontSize:16)),
             SizedBox(height: 10,),
             RaisedButton(child:Text(api_retry[PIA_Lang.lang_id]),color: Colors.white70,
               onPressed: (){
                 print("Retry");
                 reload();
               },),
          ]));
     }
  }

  Widget Footer()
  {  if(is_loading) {
    return Positioned(
        child: new Align(alignment: FractionalOffset.bottomCenter,
          child: Container(
              padding: EdgeInsets.all(10),
              child: SizedBox(
                  child:
                  new CircularProgressIndicator(
                      valueColor: new AlwaysStoppedAnimation(
                          Colors.black),
                      strokeWidth: 2.0),
                  height: 15.0,
                  width: 15.0)),
        ));
   }
  else
     return  Container();
  }

  @override
  void API_Error(DioErrorType error) {
    // TODO: implement API_Error
  }

  @override
  void API_Success(Map<String, dynamic> json, List j) {
    print("Table API Success " + j.length.toString() + " / pagesize:" + widget.pagesize.toString());
    if(j.length < widget.pagesize)
    {  all_loaded = true; }
    else
    {  pageno = pageno + 1; }

    is_loading = false;
    firstload = false;

    setState(() {
      j.forEach((dynamic value) {
        items.add(value);
      });
      print("Item count "+ items.length.toString());

    });
  }

  @override
  void API_Upload_prodcess(int upload, int total) {
    // TODO: implement API_Upload_prodcess
  }

}

==> 0.1.0.tar.gz: lib/API/PIA_API_Gridlist.dart <==
import 'package:dio/src/dio_error.dart';
import 'package:flutter/material.dart';
import 'package:pia_framework_base/API/PIA_API_Background.dart';
import 'package:pia_framework_base/API/PIA_API_Widget.dart';
import 'package:pia_framework_base/Variable/PIA_Global.dart';
import 'package:pia_framework_base/Lang/PIA_Lang.dart';

class PIA_API_Gridlist_Controller
{   PIA_API_GridList_State list_state;
    void reload(String url, Map<String,String> post)
      {  list_state.Url_reload(url,post); }
}


class PIA_API_Gridlist_CallBack
{    Widget Gridlist_item(int item,Map<String,dynamic> sdata,Function updatestate) {}

/* example for callback
 return new Padding(padding: EdgeInsets.all(0),
      child: new ListTile(
      //leading: PIA_Image("Asset/Image/company.png",50,50,0,BoxFit.cover),
      title: new Text(sdata["api"],style:new TextStyle(color: (sdata["isclick"] == "1" ) ? Colors.blue : Colors.pink)),
      subtitle: (sdata["isclick"] == "1") ? PIA_Image("Asset/Image/company.png",150,150,0,BoxFit.cover): null,
          onTap: (){
        if(sdata["isclick"] != "1")
          {  sdata["isclick"] = "1"; }
         else
          {  sdata["isclick"] = ""; }

        updatestate(null);
      },
    ));
*/
}

class PIA_API_GridList extends StatefulWidget
{  String url;
   Map<String,String> post;
   PIA_API_Gridlist_CallBack callback;
   PIA_API_Gridlist_Controller controller;

   int pagesize = 20;

   PIA_API_GridList({Key key, this.url,this.post,this.callback,this.pagesize,this.controller}) : super(key: key);

   @override
   State<StatefulWidget> createState() {
     // TODO: implement createState

     if(post == null)
       { post = Map<String,String>(); }

     if(pagesize == null)
       { pagesize = 20; }

     post["pageno"] = "1";
     post["pagesize"] = "$pagesize";

     print("PIA_TableList Build " + post.toString());

     var tablestate = PIA_API_GridList_State();
     controller.list_state = tablestate;
     return tablestate;
  }
}


class PIA_API_GridList_State  extends  State<PIA_API_GridList> implements PIA_API_CallBack{

  bool firstload = true;
  bool all_loaded = false;
  bool is_loading = false;
  bool is_error = false;

  ScrollController _controller;
  int pageno = 1;


  final items = new List<Map<String,dynamic>>();

  void initState() {
    _controller = ScrollController();
    _controller.addListener(_scrollListener);
  }

  _scrollListener() {
    if (_controller.offset >= _controller.position.maxScrollExtent &&
        !_controller.position.outOfRange) {
        setState(() {
          loadurl();
        });
    }
    if (_controller.offset <= _controller.position.minScrollExtent &&
        !_controller.position.outOfRange) {
    }
  }

  void Url_reload(String url,Map<String,String> post)
  {  PIA_API_Background.Stop();
  widget.post = post;
  widget.url = url;
  is_loading = false;
  reload();
  }


  void reload()
  { if(is_loading) return;

    setState(()
    { items.clear();
      firstload = true;
      all_loaded = false;
      is_loading = false;
      pageno = 1;

      widget.post["pageno"] = "$pageno";
      widget.post["pagesize"] = "${widget.pagesize}";

     });
  }

  void loadurl()
  {  print("Load URL ${is_loading}");

     if(!all_loaded && !is_loading) {
     is_loading = true;
     widget.post["pageno"] = "$pageno";
     widget.post["pagesize"] = "${widget.pagesize}";
     PIA_API_Background.Post(widget.url,widget.post,this);
    }
    else
     { print("All Data Loaded"); }
  }

  void update_state(Function itemfunction)
  {   setState(() {
         if(itemfunction != null)
           { itemfunction(); }
      });
  }

  @override
  void dispose() {
    super.dispose();
    PIA_API_Background.Stop();
    print("TableView dispose");
  }



  @override
  Widget build(BuildContext context) {
    print("Tablelist Build firstload $firstload "+ " post:" + widget.post.toString());

    if(firstload) {
        is_loading = true;
        return PIA_API_Widget(url:widget.url,post:widget.post,callback: this);
    }

    if(items.length > 0) {
      return
        new Stack(
          children: <Widget>[
               RefreshIndicator(
               onRefresh: (){ reload(); },
                child: GridView.builder(
                itemCount: items.length,
                 controller: _controller,
                 gridDelegate: new SliverGridDelegateWithFixedCrossAxisCount(crossAxisCount:2),
                  itemBuilder: (BuildContext context, int index) {
                      return widget.callback.Gridlist_item(index, items[index],update_state);
                  },
           )),
            Footer(),
          ]);
    }
    else {
      return Center(
          child: Column(
          mainAxisAlignment: MainAxisAlignment.center,
          crossAxisAlignment: CrossAxisAlignment.center,
          children: <Widget>[
             Text(tableview_nodata[PIA_Lang.lang_id],style:TextStyle(fontSize:16)),
             SizedBox(height: 10,),
             RaisedButton(child:Text(api_retry[PIA_Lang.lang_id]),color: Colors.white70,
               onPressed: (){
                 print("Retry");
                 reload();
               },),
          ]));
     }
  }

  Widget Footer()
  {  if(is_loading) {
    return Positioned(
        child: new Align(alignment: FractionalOffset.bottomCenter,
          child: Container(
              padding: EdgeInsets.all(10),
              child: SizedBox(
                  child:
                  new CircularProgressIndicator(
                      valueColor: new AlwaysStoppedAnimation(
                          Colors.black),
                      strokeWidth: 2.0),
                  height: 15.0,
                  width: 15.0)),
        ));
   }
  else
     return  Container();
  }

  @override
  void API_Error(DioErrorType error) {
    // TODO: implement API_Error
  }

  @override
  void API_Success(Map<String, dynamic> json, List j) {
    print("Table API Success " + j.length.toString() + " / pagesize:" + widget.pagesize.toString());
    if(j.length < widget.pagesize)
    {  all_loaded = true; }
    else
    {  pageno = pageno + 1; }

    is_loading = false;
    firstload = false;

    setState(() {
      j.forEach((dynamic value) {
        items.add(value);
      });
      print("Item count "+ items.length.toString());

    });
  }

  @override
  void API_Upload_prodcess(int upload, int total) {
    // TODO: implement API_Upload_prodcess
  }

}
